"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _assign = require("babel-runtime/core-js/object/assign");

var _assign2 = _interopRequireDefault(_assign);

var _keys = require("babel-runtime/core-js/object/keys");

var _keys2 = _interopRequireDefault(_keys);

var _asyncToGenerator2 = require("babel-runtime/helpers/asyncToGenerator");

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _capitalize = require("capitalize");

var _capitalize2 = _interopRequireDefault(_capitalize);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*  the mixin class  */
class gtsUtilSequelizeFields {
    /*  update all relation fields of an entity  */
    _entityUpdateFields(type, obj, def, upd, ctx, info) {
        var _this = this;

        return (0, _asyncToGenerator3.default)(function* () {
            /*  determine common Sequelize options  */
            let opts = {};
            if (ctx.tx !== undefined) opts.transaction = ctx.tx;

            /*  iterate over all relationships...  */
            let rels = (0, _keys2.default)(upd);
            for (let i = 0; i < rels.length; i++) {
                let name = rels[i];

                /*  determine target type and relationship cardinality  */
                let t = info.schema._typeMap[type]._fields[name].type;
                let many = false;
                while (typeof t.ofType === "object") {
                    if (t.constructor.name === "GraphQLList") many = true;
                    t = t.ofType;
                }
                let target = t.name;

                /*  helper method for changing a single relationship  */
                const changeRelation = (() => {
                    var _ref = (0, _asyncToGenerator3.default)(function* (prefix, ids) {
                        /*  map all ids onto real ORM objects  */
                        let opts2 = (0, _assign2.default)(opts, { where: { id: ids } });
                        let objs = yield _this._models[target].findAll(opts2);

                        /*  sanity check requested ids  */
                        if (objs.length < ids.length) {
                            let found = {};
                            objs.forEach(function (obj) {
                                found[obj.id] = true;
                            });
                            for (let j = 0; j < ids.length; j++) if (!found[ids[j]]) throw new Error(`no such entity ${target}#${ids[j]} found`);
                        }

                        /*  sanity check usage  */
                        if (!many && ids.length > 1) throw new Error(`relationship ${name} on type ${type} has cardinality 0..1 ` + "and cannot receive more than one foreign entity");

                        /*  change relationship  */
                        if (many) {
                            /*  change relationship of cardinality 0..N  */
                            let method = `${prefix}${(0, _capitalize2.default)(name)}`;
                            if (typeof obj[method] !== "function") throw new Error("relationship mutation method not found " + `to ${prefix} relation ${name} on type ${type}`);
                            yield obj[method](objs, opts);
                        } else {
                            /*  change relationship of cardinality 0..1  */
                            if (prefix === "add") prefix = "set";
                            let method = `${prefix}${(0, _capitalize2.default)(name)}`;
                            if (typeof obj[method] !== "function") throw new Error("relationship mutation method not found " + `to ${prefix} relation ${name} on type ${type}`);
                            let relObj = prefix !== "remove" ? objs.length ? objs[0] : null : null;
                            yield obj[method](relObj, opts);
                        }
                    });

                    return function changeRelation(_x, _x2) {
                        return _ref.apply(this, arguments);
                    };
                })();

                /*  determine relationship value and dispatch according to operation  */
                let value = upd[name];
                if (value.set) yield changeRelation("set", value.set);
                if (value.del) yield changeRelation("remove", value.del);
                if (value.add) yield changeRelation("add", value.add);
            }
        })();
    }

    /*  map Sequelize "undefined" values to GraphQL "null" values to
        ensure that the GraphQL engine does not complain about resolvers
        which return "undefined" for "null" values.  */
    _mapFieldValues(type, obj, ctx, info) {
        /*  determine allowed fields  */
        let allowed = this._fieldsOfGraphQLType(info, type);

        /*  iterate over all GraphQL attributes  */
        (0, _keys2.default)(allowed.attribute).forEach(attribute => {
            /*  map Sequelize "undefined" to GraphQL "null"  */
            if (obj[attribute] === undefined) obj[attribute] = null;
        });
    }
}
exports.default = gtsUtilSequelizeFields; /*
                                          **  GraphQL-Tools-Sequelize -- Integration of GraphQL-Tools and Sequelize ORM
                                          **  Copyright (c) 2016-2017 Ralf S. Engelschall <rse@engelschall.com>
                                          **
                                          **  Permission is hereby granted, free of charge, to any person obtaining
                                          **  a copy of this software and associated documentation files (the
                                          **  "Software"), to deal in the Software without restriction, including
                                          **  without limitation the rights to use, copy, modify, merge, publish,
                                          **  distribute, sublicense, and/or sell copies of the Software, and to
                                          **  permit persons to whom the Software is furnished to do so, subject to
                                          **  the following conditions:
                                          **
                                          **  The above copyright notice and this permission notice shall be included
                                          **  in all copies or substantial portions of the Software.
                                          **
                                          **  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
                                          **  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
                                          **  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
                                          **  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
                                          **  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
                                          **  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
                                          **  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
                                          */

/*  external dependencies  */
